@@ -29,13 +29,13 @@ from optparse import OptionParser
 import output
 import shell
 import yum
+from yum.constants import *
 import yum.Errors
 import yum.misc
 import rpmUtils.arch
 from rpmUtils.miscutils import compareEVR
-from yum.packages import parsePackages, returnBestPackages, YumInstalledPackage, YumLocalPackage
-from yum.logger import Logger
-from yum.config import yumconf
+from yum.packages import parsePackages, YumInstalledPackage, YumLocalPackage
+from yum.logger import Logger, SysLogger, LogContainer
 from yum import pgpmsg
 from i18n import _
 import callback
@@ -43,7 +43,7 @@ import urlgrabber
 import urlgrabber.grabber
 
 class CliError(yum.Errors.YumBaseError):
-   def __init__(self, args=None):
+   def __init__(self, args=''):
         yum.Errors.YumBaseError.__init__(self)
         self.args = args
 
@@ -53,9 +53,16 @@ class YumBaseCli(yum.YumBase, output.YumOutput):
        
     def __init__(self):
         yum.YumBase.__init__(self)
-        self.localPackages = [] # for local package handling - possibly needs
-                                # to move to the lower level class
-
+        self.in_shell = False
+        self.yum_cli_commands = ['update', 'install','info', 'list', 'erase',
+                                'grouplist', 'groupupdate', 'groupinstall',
+                                'groupremove', 'groupinfo', 'makecache',
+                                'clean', 'remove', 'provides', 'check-update',
+                                'search', 'upgrade', 'whatprovides',
+                                'localinstall', 'localupdate',
+                                'resolvedep', 'shell', 'deplist']
+        
+        
     def doRepoSetup(self, thisrepo=None, dosack=1):
         """grabs the repomd.xml for each enabled repository 
            and sets up the basics of the repository"""
@@ -173,17 +180,17 @@ yum [options] < update | install | info | remove | list |
                 sys.exit(1)
                 
             # Initialise logger object
-            self.log=Logger(threshold=self.conf.getConfigOption('debuglevel'), 
+            self.log=Logger(threshold=self.conf.debuglevel,
                     file_object=sys.stdout)
 
             # Setup debug and error levels
             if opts.debuglevel is not None:
                 self.log.threshold=opts.debuglevel
-                self.conf.setConfigOption('debuglevel', opts.debuglevel)
+                self.conf.debuglevel = opts.debuglevel
 
             if opts.errorlevel is not None:
                 self.errorlog.threshold=opts.errorlevel
-                self.conf.setConfigOption('errorlevel', opts.errorlevel)
+                self.conf.errorlevel = opts.errorlevel
 
         except ValueError, e:
             self.errorlog(0, _('Options Error: %s') % e)
@@ -211,13 +218,13 @@ yum [options] < update | install | info | remove | list |
             # set some things in it.
                 
             # who are we:
-            self.conf.setConfigOption('uid', os.geteuid())
+            self.conf.uid = os.geteuid()
 
             # version of yum
-            self.conf.setConfigOption('yumversion', yum.__version__)
+            self.conf.yumversion = yum.__version__
             
             # syslog-style log
-            if self.conf.getConfigOption('uid') == 0:
+            if self.conf.uid == 0:
                 logpath = os.path.dirname(self.conf.logfile)
                 if not os.path.exists(logpath):
                     try:
@@ -234,18 +241,26 @@ yum [options] < update | install | info | remove | list |
 
                 logfile =  os.fdopen(logfd, 'a')
                 fcntl.fcntl(logfd, fcntl.F_SETFD)
-                self.filelog = Logger(threshold = 10, file_object = logfile, 
+            
+                                      
+                filelog_object = Logger(threshold = 10, file_object = logfile, 
                                 preprefix = self.printtime)
             else:
-                self.filelog = Logger(threshold = 10, file_object = None, 
+                filelog_object = Logger(threshold = 10, file_object = None, 
                                 preprefix = self.printtime)
+
+            syslog_object = SysLogger(threshold = 10, 
+                                      facility=self.conf.syslog_facility,
+                                      ident='yum')
             
+            self.filelog = LogContainer([syslog_object, filelog_object])
+
             # Handle remaining options
             if opts.assumeyes:
-                self.conf.setConfigOption('assumeyes',1)
+                self.conf.assumeyes =1
 
             if opts.cacheonly:
-                self.conf.setConfigOption('cache', 1)
+                self.conf.cache = 1
 
             if opts.sleeptime is not None:
                 sleeptime = random.randrange(opts.sleeptime*60)
@@ -253,16 +268,16 @@ yum [options] < update | install | info | remove | list |
                 sleeptime = 0
 
             if opts.obsoletes:
-                self.conf.setConfigOption('obsoletes', 1)
+                self.conf.obsoletes = 1
 
             if opts.installroot:
-                self.conf.setConfigOption('installroot', opts.installroot)
+                self.conf.installroot = opts.installroot
 
             for exclude in opts.exclude:
                 try:
-                    excludelist = self.conf.getConfigOption('exclude')
+                    excludelist = self.conf.exclude
                     excludelist.append(exclude)
-                    self.conf.setConfigOption('exclude', excludelist)
+                    self.conf.exclude = excludelist
                 except yum.Errors.ConfigError, e:
                     self.errorlog(0, _(e))
                     self.usage()
@@ -302,9 +317,8 @@ yum [options] < update | install | info | remove | list |
             sys.exit(1)
     
         # set our caching mode correctly
-        
-        if self.conf.getConfigOption('uid') != 0:
-            self.conf.setConfigOption('cache', 1)
+        if self.conf.uid != 0:
+            self.conf.cache = 1
         # run the sleep - if it's unchanged then it won't matter
         time.sleep(sleeptime)
 
@@ -313,13 +327,13 @@ yum [options] < update | install | info | remove | list |
         """reads self.cmds and parses them out to make sure that the requested 
         base command + argument makes any sense at all""" 
 
-        self.log(3, 'Yum Version: %s' % self.conf.getConfigOption('yumversion'))
+        self.log(3, 'Yum Version: %s' % self.conf.yumversion)
         self.log(3, 'COMMAND: %s' % self.cmdstring)
-        self.log(3, 'Installroot: %s' % self.conf.getConfigOption('installroot'))
-        if len(self.conf.getConfigOption('commands')) == 0 and len(self.cmds) < 1:
-            self.cmds = self.conf.getConfigOption('commands')
+        self.log(3, 'Installroot: %s' % self.conf.installroot)
+        if len(self.conf.commands) == 0 and len(self.cmds) < 1:
+            self.cmds = self.conf.commands
         else:
-            self.conf.setConfigOption('commands', self.cmds)
+            self.conf.commands = self.cmds
         if len(self.cmds) < 1:
             self.errorlog(0, _('You need to give some command'))
             self.usage()
@@ -333,17 +347,11 @@ yum [options] < update | install | info | remove | list |
             for arg in self.extcmds:
                 self.log(3, '   %s' % arg)
         
-        if self.basecmd not in ['update', 'install','info', 'list', 'erase',
-                                'grouplist', 'groupupdate', 'groupinstall',
-                                'groupremove', 'groupinfo', 'makecache',
-                                'clean', 'remove', 'provides', 'check-update',
-                                'search', 'upgrade', 'whatprovides',
-                                'localinstall', 'localupdate',
-                                'resolvedep', 'shell', 'deplist']:
+        if self.basecmd not in self.yum_cli_commands:
             self.usage()
             raise CliError
     
-        if self.conf.getConfigOption('uid') != 0:
+        if self.conf.uid != 0:
             if self.basecmd in ['install', 'update', 'clean', 'upgrade','erase', 
                                 'groupupdate', 'groupinstall', 'remove',
                                 'groupremove', 'importkey', 'makecache', 
@@ -431,8 +439,10 @@ For more information contact your distribution or package provider.
         """do a shell-like interface for yum commands"""
 
         self.log(2, 'Setting up Yum Shell')
+        self.in_shell = True
         self.doTsSetup()
         self.doRpmDBSetup()
+        
         if len(self.extcmds) == 0:
             yumshell = shell.YumShell(base=self)
             yumshell.cmdloop()
@@ -474,7 +484,7 @@ For more information contact your distribution or package provider.
 
             
         elif self.basecmd == 'upgrade':
-            self.conf.setConfigOption('obsoletes', 1)
+            self.conf.obsoletes = 1
             self.log(2, "Setting up Upgrade Process")
             try:
                 return self.updatePkgs()
@@ -542,7 +552,7 @@ For more information contact your distribution or package provider.
               return 1, [str(e)]
 
         elif self.basecmd == 'clean':
-            self.conf.setConfigOption('cache', 1)
+            self.conf.cache = 1
             return self.cleanCli()
         
         elif self.basecmd in ['groupupdate', 'groupinstall', 'groupremove', 
@@ -561,18 +571,12 @@ For more information contact your distribution or package provider.
             if self.basecmd == 'grouplist':
                 return self.returnGroupLists()
             
-            elif self.basecmd == 'groupinstall':
+            elif self.basecmd in ['groupinstall', 'groupupdate']:
                 try:
                     return self.installGroups()
                 except yum.Errors.YumBaseError, e:
                     return 1, [str(e)]
             
-            elif self.basecmd == 'groupupdate':
-                try:
-                    return self.updateGroups()
-                except yum.Errors.YumBaseError, e:
-                    return 1, [str(e)]
-            
             elif self.basecmd == 'groupremove':
                 try:
                     return self.removeGroups()
@@ -610,6 +614,8 @@ For more information contact your distribution or package provider.
             self.log(2, "This may take a while depending on the speed of this computer")
             self.log(3, '%s' % self.pickleRecipe())
             try:
+                for repo in self.repos.findRepos('*'):
+                    repo.metadata_expire = 0
                 self.doRepoSetup(dosack=0)
                 self.repos.populateSack(with='metadata', pickleonly=1)
                 self.repos.populateSack(with='filelists', pickleonly=1)
@@ -631,15 +637,18 @@ For more information contact your distribution or package provider.
         
         # Check which packages have to be downloaded
         downloadpkgs = []
+        stuff_to_download = False
         for txmbr in self.tsInfo.getMembers():
             if txmbr.ts_state in ['i', 'u']:
+                stuff_to_download = True
                 po = txmbr.po
                 if po:
                     downloadpkgs.append(po)
 
         # Report the total download size to the user, so he/she can base
         # the answer on this info
-        self.reportDownloadSize(downloadpkgs)
+        if stuff_to_download:
+            self.reportDownloadSize(downloadpkgs)
         
         # confirm with user
         if self._promptWanted():
@@ -666,7 +675,7 @@ For more information contact your distribution or package provider.
         self.log(2, 'Running Transaction Test')
         tsConf = {}
         for feature in ['diskspacecheck']: # more to come, I'm sure
-            tsConf[feature] = self.conf.getConfigOption(feature)
+            tsConf[feature] = getattr(self.conf, feature)
         
         testcb = callback.RPMInstallCallback(output=0)
         testcb.tsInfo = self.tsInfo
@@ -734,7 +743,7 @@ For more information contact your distribution or package provider.
                 # Bail if not -y and stdin isn't a tty as key import will
                 # require user confirmation
                 if not sys.stdin.isatty() and not \
-                            self.conf.getConfigOption('assumeyes'):
+                            self.conf.assumeyes:
                     raise yum.Errors.YumBaseError, \
                         'Refusing to automatically import keys when running ' \
                         'unattended.\nUse "-y" to override.'
@@ -774,7 +783,7 @@ For more information contact your distribution or package provider.
 
                     # Try installing/updating GPG key
                     self.log(0, 'Importing GPG key 0x%s "%s"' % (hexkeyid, userid))
-                    if not self.conf.getConfigOption('assumeyes'):
+                    if not self.conf.assumeyes:
                         if not self.userconfirm():
                             self.log(0, 'Exiting on user command')
                             return 1
@@ -898,26 +907,27 @@ For more information contact your distribution or package provider.
                                         # does not include a version-rel section
                             if pkg.pkgtup in exactmatch:
                                 if not toBeInstalled.has_key(pkg.name): toBeInstalled[pkg.name] = []
-                                toBeInstalled[pkg.name].append(pkg.pkgtup)
+                                toBeInstalled[pkg.name].append(pkg)
                 else: # we've not got any installed that match n or n+a
                     self.log(4, 'No other %s installed, adding to list for potential install' % pkg.name)
                     if not toBeInstalled.has_key(pkg.name): toBeInstalled[pkg.name] = []
-                    toBeInstalled[pkg.name].append(pkg.pkgtup)
+                    toBeInstalled[pkg.name].append(pkg)
         
         
         # this is where I could catch the installs of compat and multilib 
         # arches on a single yum install command. 
-        pkglist = returnBestPackages(toBeInstalled)
-        
+        pkglist = []
+        for name in toBeInstalled.keys():
+            pkglist.extend(self.bestPackagesFromList(toBeInstalled[name]))
+            
         # This is where we need to do a lookup to find out if this install
         # is also an obsolete. if so then we need to mark it as such in the
         # tsInfo.
         if len(pkglist) > 0:
             self.log(3, 'reduced installs :')
-        for pkgtup in pkglist:
-            self.log(3,'   %s.%s %s:%s-%s' % pkgtup)
-            po = self.getPackageObject(pkgtup)
-            self.tsInfo.addInstall(po)
+        for po in pkglist:
+            self.log(3,'   %s.%s %s:%s-%s' % po.pkgtup)
+            self.install(po)
 
         if len(passToUpdate) > 0:
             self.log(3, 'potential updates :')
@@ -950,7 +960,7 @@ For more information contact your distribution or package provider.
         installed = self.rpmdb.getPkgList()
         self.doUpdateSetup()
         updates = self.up.getUpdatesTuples()
-        if self.conf.getConfigOption('obsoletes'):
+        if self.conf.obsoletes:
             obsoletes = self.up.getObsoletesTuples(newest=1)
         else:
             obsoletes = []
@@ -965,7 +975,7 @@ For more information contact your distribution or package provider.
             for (new, old) in updates:
                 txmbrs = self.tsInfo.getMembers(pkgtup=old)
 
-                if txmbrs and txmbrs[0].output_state == 'obsoleted':
+                if txmbrs and txmbrs[0].output_state == TS_OBSOLETED: 
                     self.log(5, 'Not Updating Package that is already obsoleted: %s.%s %s:%s-%s' % old)
                 else:
                     updating_pkg = self.getPackageObject(new)
@@ -1052,7 +1062,7 @@ For more information contact your distribution or package provider.
                     erases.extend(depmatches)
             
         for pkg in erases:
-            self.tsInfo.addErase(pkg)
+            self.remove(po=pkg)
         
         if len(self.tsInfo) > oldcount:
             change = len(self.tsInfo) - oldcount
@@ -1124,22 +1134,36 @@ For more information contact your distribution or package provider.
                     donothingpkgs.append(po)
                     continue
 
+        # handle excludes for a localinstall
+        toexc = []
+        if len(self.conf.exclude) > 0:
+           exactmatch, matched, unmatched = \
+                   parsePackages(installpkgs + map(lambda x: x[0], updatepkgs),
+                                 self.conf.exclude, casematch=1)
+           toexc = exactmatch + matched
 
         for po in installpkgs:
+            if po in toexc:
+               self.log(3, 'Excluding %s' % po)
+               continue
+            
             self.log(2, 'Marking %s to be installed' % po.localpath)
             self.localPackages.append(po)
-            self.tsInfo.addInstall(po)
+            self.install(po=po)
         
         for (po, oldpo) in updatepkgs:
+            if po in toexc:
+               self.log(3, 'Excluding %s' % po)
+               continue
+           
             self.log(2, 'Marking %s as an update to %s' % (po.localpath, oldpo))
             self.localPackages.append(po)
             self.tsInfo.addUpdate(po, oldpo)
         
         for po in donothingpkgs:
             self.log(2, '%s: does not update installed package.' % po.localpath)
-        
+
         if len(self.tsInfo) > oldcount:
-            
             return 2, ['Package(s) to install']
         return 0, ['Nothing to do']
         
@@ -1205,9 +1229,14 @@ For more information contact your distribution or package provider.
             args = self.extcmds
             
         searchlist = ['name', 'summary', 'description', 'packager', 'group', 'url']
-        matching = self.searchPackages(searchlist, args, callback=self.matchcallback)
-
-        if len(matching.keys()) == 0:
+        matching = self.searchGenerator(searchlist, args)
+        
+        total = 0
+        for (po, matched_value) in matching:
+            self.matchcallback(po, matched_value)
+            total += 1
+            
+        if total == 0:
             return 0, ['No Matches found']
         return 0, []
 
@@ -1311,12 +1340,12 @@ For more information contact your distribution or package provider.
         if len(installed) > 0:
             self.log(2, 'Installed Groups:')
             for group in installed:
-                self.log(2, '   %s' % group)
+                self.log(2, '   %s' % group.name)
         
         if len(available) > 0:
             self.log(2, 'Available Groups:')
             for group in available:
-                self.log(2, '   %s' % group)
+                self.log(2, '   %s' % group.name)
 
             
         return 0, ['Done']
@@ -1326,141 +1355,73 @@ For more information contact your distribution or package provider.
         if userlist is None:
             userlist = self.extcmds
         
-        for group in userlist:
-            if self.groupInfo.groupExists(group):
+        for strng in userlist:
+            group = self.comps.return_group(strng)
+            if group:
                 self.displayPkgsInGroups(group)
             else:
-                self.errorlog(1, 'Warning: Group %s does not exist.' % group)
+                self.errorlog(1, 'Warning: Group %s does not exist.' % strng)
         
         return 0, []
         
     def installGroups(self, grouplist=None):
-        """for each group requested attempt to install all pkgs/metapkgs of default
-           or mandatory. Also recurse lists of groups to provide for them too."""
-        
-        if grouplist is None:
-            grouplist = self.extcmds
+        """for each group requested do 'selectGroup' on them."""
         
         self.doRepoSetup()
-        pkgs = [] # package objects to be installed
-        installed = self.rpmdb.getPkgList()
-        availablepackages = {}
-        for po in self.pkgSack.returnPackages():
-            if po.pkgtup not in installed:
-                    availablepackages[po.name] = 1
-
-        for group in grouplist:
-            if not self.groupInfo.groupExists(group):
-                self.errorlog(0, _('Warning: Group %s does not exist.') % group)
-                continue
-            pkglist = self.groupInfo.pkgTree(group)
-            for pkg in pkglist:
-                if availablepackages.has_key(pkg):
-                    pkgs.append(pkg)
-                    self.log(4, 'Adding package %s for groupinstall of %s.' % (pkg, group))
-
-        if len(pkgs) > 0:
-            self.log(2, 'Passing package list to Install Process')
-            self.log(4, 'Packages being passed:')
-            for pkg in pkgs:
-                self.log(4, '%s' % pkg)
-            return self.installPkgs(userlist=pkgs)
-        else:
-            return 0, ['No packages in any requested group available to install']
-
-    
-    def updateGroups(self, grouplist=None):
-        """get list of any pkg in group that is installed, check to update it
-           get list of any mandatory or default pkg attempt to update it if it is installed
-           or install it if it is not installed"""
-
+        pkgs_used = []
+        
         if grouplist is None:
             grouplist = self.extcmds
         
-        if len(grouplist) == 0:
-            self.usage()
+        for group_string in grouplist:
+            group = self.comps.return_group(group_string)
+            if not group:
+                self.errorlog(0, _('Warning: Group %s does not exist.') % group)
+                continue
             
-        self.doRepoSetup()
-        self.doUpdateSetup()
-        
-        grouplist.sort()
-        updatesbygroup = []
-        installsbygroup = []
-        updateablenames = []
-        
-        for group in grouplist:
-            if not self.groupInfo.groupExists(group):
+            try:
+                txmbrs = self.selectGroup(group.groupid)
+            except yum.Errors.GroupsError, e:
                 self.errorlog(0, _('Warning: Group %s does not exist.') % group)
                 continue
-
-            required = self.groupInfo.requiredPkgs(group)
-            all = self.groupInfo.pkgTree(group)
-            for pkgn in all:
-                if self.rpmdb.installed(name=pkgn):
-                    if len(self.up.getUpdatesList(name=pkgn)) > 0:
-                        updatesbygroup.append((group, pkgn))
-                else:
-                    if pkgn in required:
-                        installsbygroup.append((group, pkgn))
-        
-        updatepkgs = []
-        installpkgs = []
-        for (group, pkg) in updatesbygroup:
-            self.log(2, _('From %s updating %s') % (group, pkg))
-            updatepkgs.append(pkg)
-        for (group, pkg) in installsbygroup:
-            self.log(2, _('From %s installing %s') % (group, pkg))
-            installpkgs.append(pkg)
-
-        
-
-        if len(installpkgs) > 0:
-            self.installPkgs(userlist=installpkgs)
-        
-        if len(updatepkgs) > 0:
-            self.updatePkgs(userlist=updatepkgs, quiet=1)
-        
-        if len(self.tsInfo) > 0:
-            return 2, ['Group updating']
+            else:
+                pkgs_used.extend(txmbrs)
+            
+        if not pkgs_used:
+            return 0, ['No packages in any requested group available to install or update']
         else:
-            return 0, [_('Nothing in any group to update or install')]
-    
+            return 2, ['%d Package(s) to Install' % len(pkgs_used)]
+
     def removeGroups(self, grouplist=None):
         """Remove only packages of the named group(s). Do not recurse."""
 
+        pkgs_used = []
+        
         if grouplist is None:
             grouplist = self.extcmds
         
-        if len(grouplist) == 0:
-            self.usage()
-        
         erasesbygroup = []
-        for group in grouplist:
-            if not self.groupInfo.groupExists(group):
-                self.errorlog(0, _('Warning: Group %s does not exist.') % group)
+        for group_string in grouplist:
+            try:
+                txmbrs = self.groupRemove(group_string)
+            except yum.Errors.GroupsError, e:
+                self.errorlog(0, 'No group named %s exists' % group_string)
                 continue
-        
-            allpkgs = self.groupInfo.allPkgs(group)
-            for pkg in allpkgs:
-                if self.rpmdb.installed(name=pkg):
-                    erasesbygroup.append((group, pkg))
-
-        erases = []
-        for (group, pkg) in erasesbygroup:
-            self.log(2, _('From %s removing %s') % (group, pkg))
-            erases.append(pkg)
-        
-        if len(erases) > 0:
-            return self.erasePkgs(userlist=erases)
-        else:
+            else:
+                pkgs_used.extend(txmbrs)
+                
+        if not pkgs_used:
             return 0, ['No packages to remove from groups']
+        else:
+            return 2, ['%d Package(s) to remove' % len(pkgs_used)]
+
 
 
     def _promptWanted(self):
         # shortcut for the always-off/always-on options
-        if self.conf.getConfigOption('assumeyes'):
+        if self.conf.assumeyes:
             return False
-        if self.conf.getConfigOption('alwaysprompt'):
+        if self.conf.alwaysprompt:
             return True
         
         # prompt if:
@@ -1479,8 +1440,15 @@ For more information contact your distribution or package provider.
     def usage(self):
         '''Print out command line usage
         '''
-        print
-        self.optparser.print_help()
+        if not self.in_shell:
+            print 
+            self.optparser.print_help()
+        else:
+            print 
+            self.optparser.print_usage()
+            
+            
+            
 
 class YumOptionParser(OptionParser):
     '''Subclass that makes some minor tweaks to make OptionParser do things the
@@ -1498,6 +1466,8 @@ class YumOptionParser(OptionParser):
         self.base.errorlog(0, "Command line error: "+msg)
         sys.exit(1)
 
+
+        
 def _filtercmdline(novalopts, valopts, args):
     '''Keep only specific options from the command line argument list
 
