@@ -41,7 +41,13 @@ arches = {
     "sparc64": "sparcv9",
     "sparcv9": "sparcv8",
     "sparcv8": "sparc",
-    "sparc": "noarch"
+    "sparc": "noarch",
+
+    # alpha
+    "alphaev6": "alphaev56",
+    "alphaev56": "alphaev5",
+    "alphaev5": "alpha",
+    "alpha": "noarch",
     }
 
 # this computes the difference between myarch and targetarch
@@ -240,6 +246,11 @@ def getBaseArch(myarch=None):
 
     if not arches.has_key(myarch): # this is dumb, but <shrug>
         return myarch
+
+    if myarch == "sparc64":
+        return "sparc"
+    elif myarch.startswith("ppc64"):
+        return "ppc"
         
     if isMultiLibArch(arch=myarch):
         if multilibArches.has_key(myarch):
