@@ -33,6 +33,9 @@ class Updates:
         self.available = availlist # list of available pkgs (n, a, e, v, r)
         self.rawobsoletes = {} # dict of obsoleting package->[what it obsoletes]
         self.exactarch = 1 # don't change archs by default
+        self.exactarchlist = ['kernel', 'kernel-smp', 'glibc', 'kernel-hugemem',
+                              'kernel-enterprise', 'kernel-bigmem', 'kernel-BOOT']
+                              
         self.myarch = rpmUtils.arch.getCanonArch() # this is for debugging only 
                                                    # set this if you want to 
                                                    # test on some other arch
@@ -140,12 +143,21 @@ class Updates:
         # if you find it look for the packages they obsolete
         # 
         for pkgtup in self.rawobsoletes.keys():
+            (name, arch, epoch, ver, rel) = pkgtup
             for (obs_n, flag, (obs_e, obs_v, obs_r)) in self.rawobsoletes[(pkgtup)]:
                 if flag in [None, 0]: # unversioned obsolete
                     if self.installdict.has_key((obs_n, None)):
                         for (rpm_a, rpm_e, rpm_v, rpm_r) in self.installdict[(obs_n, None)]:
                             # make sure the obsoleting pkg is not already installed
-                            if pkgtup not in self.installed:
+                            willInstall = 1
+                            if self.installdict.has_key((name, None)):
+                                for (ins_a, ins_e, ins_v, ins_r) in self.installdict[(name, None)]:
+                                    pkgver = (epoch, ver, rel)
+                                    installedver = (ins_e, ins_v, ins_r)
+                                    if self.returnNewest((pkgver, installedver)) == installedver:
+                                        willInstall = 0
+                                        break
+                            if willInstall:
                                 if not obsdict.has_key(pkgtup):
                                     obsdict[pkgtup] = []
                                 obsdict[pkgtup].append((obs_n, rpm_a, rpm_e, rpm_v, rpm_r))
@@ -157,13 +169,33 @@ class Updates:
                                                         obs_v, obs_r)), (obs_n,\
                                                         rpm_a, rpm_e, rpm_v, rpm_r)):
                                 # make sure the obsoleting pkg is not already installed
-                                if pkgtup not in self.installed:
+                                willInstall = 1
+                                if self.installdict.has_key((name, None)):
+                                    for (ins_a, ins_e, ins_v, ins_r) in self.installdict[(name, None)]:
+                                        pkgver = (epoch, ver, rel)
+                                        installedver = (ins_e, ins_v, ins_r)
+                                        if self.returnNewest((pkgver, installedver)) == installedver:
+                                            willInstall = 0
+                                            break
+                                if willInstall:
                                     if not obsdict.has_key(pkgtup):
                                         obsdict[pkgtup] = []
                                     obsdict[pkgtup].append((obs_n, rpm_a, rpm_e, rpm_v, rpm_r))
            
         self.obsoletes = obsdict
-           
+        self.makeObsoletedDict()
+
+    def makeObsoletedDict(self):
+        """creates a dict of obsoleted packages -> [obsoleting package], this
+           is to make it easier to look up what package obsoletes what item in 
+           the rpmdb"""
+        self.obsoleted_dict = {}
+        for new in self.obsoletes.keys():
+            for old in self.obsoletes[new]:
+                if not self.obsoleted_dict.has_key(old):
+                    self.obsoleted_dict[old] = []
+                self.obsoleted_dict[old].append(new)
+    
     def doUpdates(self):
         """check for key lists as populated then commit acts of evil to
            determine what is updated and/or obsoleted, populate self.updatesdict
@@ -271,7 +303,7 @@ class Updates:
         # simple cases
         for (n, a) in simpleupdate:
             # try to be as precise as possible
-            if self.exactarch:
+            if n in self.exactarchlist:
                 if self.installdict.has_key((n, a)):
                     (rpm_e, rpm_v, rpm_r) = self.returnNewest(self.installdict[(n, a)])
                     if newpkgs.has_key((n,a)):
@@ -342,7 +374,7 @@ class Updates:
                 hipdict = self.makeNADict(highestinstalledpkgs, 0)
 
                 # now we have the two sets of pkgs
-                if self.exactarch:              
+                if n in self.exactarchlist:
                     for (n, a) in hipdict:
                         if hapdict.has_key((n, a)):
                             self.debugprint('processing %s.%s' % (n, a))
@@ -385,7 +417,19 @@ class Updates:
                    
                    
         self.updatesdict = updatedict                    
+        self.makeUpdatingDict()
         
+    def makeUpdatingDict(self):
+        """creates a dict of available packages -> [installed package], this
+           is to make it easier to look up what package  will be updating what
+           in the rpmdb"""
+        self.updating_dict = {}
+        for old in self.updatesdict.keys():
+            for new in self.updatesdict[old]:
+                if not self.updating_dict.has_key(new):
+                    self.updating_dict[new] = []
+                self.updating_dict[new].append(old)
+
     def reduceListByNameArch(self, pkglist, name=None, arch=None):
         """returns a set of pkg naevr tuples reduced based on name or arch"""
         returnlist = []
@@ -493,6 +537,19 @@ class Updates:
         returnlist = self.reduceListByNameArch(tmplist, name, arch)
         
         return returnlist
+    
+    def getObsoletedList(self, newest=0, name=None):
+        """returns a list of pkgtuples obsoleting the package in name"""
+        returnlist = []
+        for new in self.obsoletes.keys():
+            for obstup in self.obsoletes[new]:
+                (n, a, e, v, r) = obstup
+                if n == name:
+                    returnlist.append(new)
+                    continue
+        return returnlist
+
+
         
     def getOthersList(self, name=None, arch=None):
         """returns a naevr tuple of the packages that are neither installed
