@@ -29,16 +29,29 @@ class YumShell(cmd.Cmd):
         self.base = base
         self.prompt = '> '
         self.result = 0
+        self.from_file = False # if we're running from a file, set this
         self.resultmsgs = ['Leaving Shell']
         if (len(base.extcmds)) > 0:
             self.file = base.extcmds[0]
-        self.commandlist = ['clean', 'repo', 'exit', 'groupinfo',
-            'groupinstall', 'grouplist', 'groupremove', 'groupupdate',
-            'info', 'install', 'list', 'localinstall', 'repository',
-            'makecache', 'provides', 'quit', 'remove', 'run', 'search',
-            'transaction', 'ts', 'update', 'config', 'deplist']
+        self.shell_specific_commands = ['repo', 'repository', 'exit', 'quit',
+                'run', 'ts', 'transaction', 'config']
+                
+        self.commandlist = self.shell_specific_commands + self.base.yum_cli_commands
 
 
+    def _shlex_split(self, input_string):
+        """split the input using shlex rules, and error or exit accordingly"""
+        
+        inputs = []
+        try:
+            inputs = shlex.split(input_string)
+        except ValueError, e:
+            self.base.errorlog(0, 'Script Error: %s' % e)
+            if self.from_file:
+                raise Errors.YumBaseError, "Fatal error in script, exiting"
+        
+        return inputs
+        
     def script(self):
         try:
             fd = open(self.file, 'r')
@@ -46,6 +59,7 @@ class YumShell(cmd.Cmd):
             sys.exit("Error: Cannot open %s for reading")
         lines = fd.readlines()
         fd.close()
+        self.from_file = True
         for line in lines:
             self.onecmd(line)
         self.onecmd('EOF')
@@ -61,11 +75,12 @@ class YumShell(cmd.Cmd):
                 return False
             self.base.cmdstring = line
             self.base.cmdstring = self.base.cmdstring.replace('\n', '')
-            self.base.cmds = shlex.split(self.base.cmdstring)
+            self.base.cmds = self._shlex_split(self.base.cmdstring)
+
             try:
                 self.base.parseCommands()
             except Errors.YumBaseError:
-                self.do_help('')
+                pass
             else:
                 self.base.doCommands()
     
@@ -74,11 +89,14 @@ class YumShell(cmd.Cmd):
     
     def do_help(self, arg):
         msg = """
-    commands:  clean, config, exit, groupinfo, groupinstall, grouplist,
-               groupremove, groupupdate, info, install, list,
-               localinstall, makecache, provides, quit, remove, 
-               repo, run, search, transaction, update, deplist
+    Shell specific arguments:
+      config - set config options
+      repository (or repo) - enable/disable repositories
+      transaction (or ts) - list, reset or run the transaction set
+      run - run the transaction set
+      exit or quit - exit the shell
     """
+            
         if arg in ['transaction', 'ts']:
             msg = """
     %s arg
@@ -103,6 +121,9 @@ class YumShell(cmd.Cmd):
         If value is given it sets that value.
         """ % arg
         
+        else:
+            self.base.usage()
+        
         self.base.log(0, msg)
         
     def do_EOF(self, line):
@@ -153,9 +174,9 @@ class YumShell(cmd.Cmd):
         (cmd, args, line) = self.parseline(line)
         # logs
         if cmd in ['debuglevel', 'errorlevel']:
-            opts = shlex.split(args)
+            opts = self._shlex_split(args)
             if not opts:
-                self.base.log(2, '%s: %s' % (cmd, self.base.conf.getConfigOption(cmd)))
+                self.base.log(2, '%s: %s' % (cmd, getattr(self.base.conf, cmd)))
             else:
                 val = opts[0]
                 try:
@@ -163,37 +184,37 @@ class YumShell(cmd.Cmd):
                 except ValueError, e:
                     self.base.errorlog(0, 'Value %s for %s cannot be made to an int' % (val, cmd))
                     return
-                self.base.conf.setConfigOption(cmd, val)
+                setattr(self.base.conf, cmd, val)
                 if cmd == 'debuglevel':
                     self.base.log.threshold = val
                 elif cmd == 'errorlevel':
                     self.base.errorlog.threshold = val
         # bools
         elif cmd in ['gpgcheck', 'obsoletes', 'assumeyes']:
-            opts = shlex.split(args)
+            opts = self._shlex_split(args)
             if not opts:
-                self.base.log(2, '%s: %s' % (cmd, self.base.conf.getConfigOption(cmd)))
+                self.base.log(2, '%s: %s' % (cmd, getattr(self.base.conf, cmd)))
             else:
                 value = opts[0]
                 if value.lower() not in BOOLEAN_STATES:
                     self.base.errorlog(0, 'Value %s for %s is not a Boolean' % (value, cmd))
                     return False
                 value = BOOLEAN_STATES[value.lower()]
-                self.base.conf.setConfigOption(cmd, value)
+                setattr(self.base.conf, cmd, value)
                 if cmd == 'obsoletes':
                     if hasattr(self.base, 'up'): # reset the updates
                         del self.base.up
         
         elif cmd in ['exclude']:
             args = args.replace(',', ' ')
-            opts = shlex.split(args)
+            opts = self._shlex_split(args)
             if not opts:
                 msg = '%s: ' % cmd
-                msg = msg + string.join(self.base.conf.getConfigOption(cmd))
+                msg = msg + string.join(getattr(self.base.conf, cmd))
                 self.base.log(2, msg)
                 return False
             else:
-                self.base.conf.setConfigOption(cmd, opts)
+                setattr(self.base.conf, cmd, opts)
                 if hasattr(self.base, 'pkgSack'): # kill the pkgSack
                     del self.base.pkgSack
                     self.base.repos._selectSackType()
@@ -214,14 +235,16 @@ class YumShell(cmd.Cmd):
         if cmd in ['list', None]:
             if self.base.repos.repos.values():
                 self.base.log(2, '%-20.20s %-40.40s  status' % ('repo id', 'repo name'))
-            for repo in self.base.repos.repos.values():
-                if repo in self.base.repos.listEnabled():
+            repos = self.base.repos.repos.values()
+            repos.sort()
+            for repo in repos:
+                if repo in self.base.repos.listEnabled() and args in ('', 'enabled'):
                     self.base.log(2, '%-20.20s %-40.40s  enabled' % (repo, repo.name))
-                else:
+                elif args in ('', 'disabled'):
                     self.base.log(2, '%-20.20s %-40.40s  disabled' % (repo, repo.name))
         
         elif cmd == 'enable':
-            repos = shlex.split(args)
+            repos = self._shlex_split(args)
             for repo in repos:
                 try:
                     changed = self.base.repos.enableRepo(repo)
@@ -243,7 +266,7 @@ class YumShell(cmd.Cmd):
                         del self.base.up
             
         elif cmd == 'disable':
-            repos = shlex.split(args)
+            repos = self._shlex_split(args)
             for repo in repos:
                 try:
                     self.base.repos.disableRepo(repo)
