@@ -28,9 +28,10 @@ import rpm
 from repomd.packageSack import ListPackageSack
 from repomd.mdErrors import PackageSackError
 from Errors import DepError, RepoError
+from constants import *
 import packages
 
-class Depsolve:
+class Depsolve(object):
     def __init__(self):
         packages.base = self
         self.dsCallback = None
@@ -69,9 +70,8 @@ class Depsolve:
                 if globc.match(name):
                     matched = 1
             if not matched:
-                self.log(2, 'Importing Additional filelist information for dependency resolution')
-                self.repos.populateSack(with='filelists')
-                
+                self.doSackFilelistPopulate()
+            
         pkgs = self.pkgSack.searchProvides(name)
         if flags == 0:
             flags = None
@@ -111,38 +111,6 @@ class Depsolve:
         
         return 0
 
-    def handleKernelModule(self, txmbr):
-        """Figure out what special magic needs to be done to install/upgrade
-           this kernel module."""
-
-        def getKernelReqs(hdr):
-            kernels = ["kernel-%s" % a for a in rpmUtils.arch.arches.keys()]
-            reqs = []
-            names = hdr[rpm.RPMTAG_REQUIRENAME]
-            flags = hdr[rpm.RPMTAG_REQUIREFLAGS]
-            ver =   hdr[rpm.RPMTAG_REQUIREVERSION]
-            if names is not None:
-                reqs = zip(names, flags, ver)
-            return filter(lambda r: r[0] in kernels, reqs)
-
-        kernelReqs = getKernelReqs(txmbr.po.returnLocalHeader())
-        instPkgs = self.rpmdb.returnTupleByKeyword(name=txmbr.po.name)
-        for pkg in instPkgs:
-            hdr = self.rpmdb.returnHeaderByTuple(pkg)[0]
-            instKernelReqs = getKernelReqs(hdr)
-            
-            for r in kernelReqs:
-                if r in instKernelReqs:
-                    # we know that an incoming kernel module requires the
-                    # same kernel as an already installed module of the
-                    # same name.  "Upgrade" this module instead of install
-                    po = packages.YumInstalledPackage(hdr)
-                    self.tsInfo.addErase(po)
-                    self.log(4, 'Removing kernel module %s upgraded to %s' %
-                             (po, txmbr.po))
-                    break
-       
-
     def populateTs(self, test=0, keepold=1):
         """take transactionData class and populate transaction set"""
 
@@ -171,12 +139,10 @@ class Depsolve:
                 rpmfile = txmbr.po.localPkg()
                 
                 if txmbr.ts_state == 'u':
-                    if txmbr.po.name.startswith("kernel-module-"):
-                        self.handleKernelModule(txmbr)
                     if self.allowedMultipleInstalls(txmbr.po):
                         self.log(5, '%s converted to install' % (txmbr.po))
                         txmbr.ts_state = 'i'
-                        txmbr.output_state = 'installing'
+                        txmbr.output_state = TS_INSTALL
 
                 
                 self.ts.addInstall(hdr, (hdr, rpmfile), txmbr.ts_state)
@@ -211,11 +177,13 @@ class Depsolve:
             self.populateTs(test=1)
             if self.dsCallback: self.dsCallback.tscheck()
             deps = self.ts.check()
-            deps = unique(deps) # get rid of duplicate deps
+
             
             if not deps:
+                self.tsInfo.changed = False
                 return (2, ['Success - deps resolved'])
-            
+
+            deps = unique(deps) # get rid of duplicate deps            
             if deps == depscopy:
                 unresolveableloop += 1
                 self.log(5, 'Identical Loop count = %d' % unresolveableloop)
@@ -224,9 +192,15 @@ class Depsolve:
                     for deptuple in deps:
                         ((name, version, release), (needname, needversion), flags, 
                           suggest, sense) = deptuple
-                        msg = 'Package %s needs %s, this is not available.' % \
-                              (name, rpmUtils.miscutils.formatRequire(needname, 
-                                                            needversion, flags))
+                        if sense == rpm.RPMDEP_SENSE_REQUIRES:
+                            msg = 'Package %s needs %s, this is not available.' % \
+                                  (name, rpmUtils.miscutils.formatRequire(needname, 
+                                                                          needversion, flags))
+                        elif sense == rpm.RPMDEP_SENSE_CONFLICTS:
+                            msg = 'Package %s conflicts with %s.' % \
+                                  (name, rpmUtils.miscutils.formatRequire(needname, 
+                                                                          needversion, flags))
+
                         errors.append(msg)
                     CheckDeps = 0
                     break
@@ -248,6 +222,7 @@ class Depsolve:
                     # if our packageSacks aren't here, then set them up
                     if not hasattr(self, 'pkgSack'):
                         self.doRepoSetup()
+                        self.doSackSetup()
                     (checkdep, missing, conflict, errormsgs) = self._processReq(dep)
                     
                 elif sense == rpm.RPMDEP_SENSE_CONFLICTS: # conflicts - this is gonna be short :)
@@ -276,8 +251,8 @@ class Depsolve:
                 self.log(4, 'Dependency Process ending')
 
             del deps
-            
 
+        self.tsInfo.changed = False
         if len(errors) > 0:
             return (1, errors)
         if len(self.tsInfo) > 0:
@@ -388,6 +363,8 @@ class Depsolve:
     def _requiringFromInstalled(self, requiringPkg, requirement, errorlist):
         """processes the dependency resolution for a dep where the requiring 
            package is installed"""
+        
+        # FIXME - should we think about dealing exclusively in package objects?
            
         (name, arch, epoch, ver, rel) = requiringPkg
         requiringPo = self.getInstalledPackageObject(requiringPkg)
@@ -403,7 +380,7 @@ class Depsolve:
         # if it is then we need to find out what is being done to it and act accordingly
         rpmdbNames = self.rpmdb.getNamePkgList()
         needmode = None # mode in the transaction of the needed pkg (if any)
-        needpkgtup = None
+        needpo = None
         providers = []
         
         if self.cheaterlookup.has_key((needname, needflags, needversion)):
@@ -437,7 +414,10 @@ class Depsolve:
             
             if thismode is not None:
                 needmode = thismode
-                needpkgtup = insttuple
+                try:
+                    needpo = self.getInstalledPackageObject(insttuple)
+                except KeyError:
+                    needpo = self.getPackageObject(insttuple)
                 self.cheaterlookup[(needname, needflags, needversion)] = insttuple
                 self.log(5, 'Mode is %s for provider of %s: %s' % 
                             (needmode, niceformatneed, inst_str))
@@ -446,11 +426,11 @@ class Depsolve:
         self.log(5, 'Mode for pkg providing %s: %s' % (niceformatneed, needmode))
         
         if needmode in ['e']:
-                self.log(5, 'TSINFO: %s package requiring %s marked as erase' %
-                                (requiringPo, needname))
-                txmbr = self.tsInfo.addErase(requiringPo)
-                txmbr.setAsDep(pkgtup = needpkgtup)
-                checkdeps = 1
+            self.log(5, 'TSINFO: %s package requiring %s marked as erase' %
+                            (requiringPo, needname))
+            txmbr = self.tsInfo.addErase(requiringPo)
+            txmbr.setAsDep(po=needpo)
+            checkdeps = 1
         
         if needmode in ['i', 'u']:
             self.doUpdateSetup()
@@ -466,11 +446,11 @@ class Depsolve:
                 for pkgtup in obslist:
                     po = self.getPackageObject(pkgtup)
                 if po:
-                    for (new, old) in self.up.getObsoletesTuples():
+                    for (new, old) in self.up.getObsoletesTuples(): # FIXME query the obsoleting_list now?
                         if po.pkgtup == new:
                             txmbr = self.tsInfo.addObsoleting(po, requiringPo)
                             self.tsInfo.addObsoleted(requiringPo, po)
-                            txmbr.setAsDep(pkgtup = needpkgtup)
+                            txmbr.setAsDep(po=needpo)
                             self.log(5, 'TSINFO: Obsoleting %s with %s to resolve dep.' % (requiringPo, po))
                             checkdeps = 1
                             return checkdeps, missingdep 
@@ -485,14 +465,14 @@ class Depsolve:
             if len(uplist) > 0:
                 if name not in self.conf.exactarchlist:
                     pkgs = self.pkgSack.returnNewestByName(name)
-                    archs = []
+                    archs = {}
                     for pkg in pkgs:
                         (n,a,e,v,r) = pkg.pkgtup
-                        archs.append(a)
-                    a = rpmUtils.arch.getBestArchFromList(archs)
-                    po = self.pkgSack.returnNewestByNameArch((n,a))
+                        archs[a] = pkg
+                    a = rpmUtils.arch.getBestArchFromList(archs.keys())
+                    po = archs[a]
                 else:
-                    po = self.pkgSack.returnNewestByNameArch((name,arch))
+                    po = self.pkgSack.returnNewestByNameArch((name,arch))[0]
                 if po.pkgtup not in uplist:
                     po = None
 
@@ -500,7 +480,7 @@ class Depsolve:
                 for (new, old) in self.up.getUpdatesTuples():
                     if po.pkgtup == new:
                         txmbr = self.tsInfo.addUpdate(po, requiringPo)
-                        txmbr.setAsDep(pkgtup = needpkgtup)
+                        txmbr.setAsDep(po=needpo)
                         self.log(5, 'TSINFO: Updating %s to resolve dep.' % po)
                 checkdeps = 1
                 
@@ -638,7 +618,8 @@ class Depsolve:
         
                 
             
-            
+        # FIXME - why can't we look up in the transaction set for the requiringPkg
+        # and know what needs it that way and provide a more sensible dep structure in the txmbr
         if (best.name, best.arch) in self.rpmdb.getNameArchPkgList():
             self.log(3, 'TSINFO: Marking %s as update for %s' % (best, name))
             txmbr = self.tsInfo.addUpdate(best)
@@ -691,14 +672,14 @@ class Depsolve:
             if len(uplist) > 0:
                 if confname not in self.conf.exactarchlist:
                     pkgs = self.pkgSack.returnNewestByName(confname)
-                    archs = []
+                    archs = {}
                     for pkg in pkgs:
                         (n,a,e,v,r) = pkg.pkgtup
-                        archs.append(a)
-                    a = rpmUtils.arch.getBestArchFromList(archs)
-                    po = self.pkgSack.returnNewestByNameArch((n,a))
+                        archs[a] = pkg
+                    a = rpmUtils.arch.getBestArchFromList(archs.keys())
+                    po = archs[a]
                 else:
-                    po = self.pkgSack.returnNewestByNameArch((confname,confarch))
+                    po = self.pkgSack.returnNewestByNameArch((confname,confarch))[0]
                 if po.pkgtup not in uplist:
                     po = None
 
