@@ -14,13 +14,13 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 # Copyright 2004 Duke University 
 
-
-
 import os
 import os.path
-import types
 import re
+import fnmatch
 import urlparse
+import types
+import time
 
 import Errors
 from urlgrabber.grabber import URLGrabber
@@ -31,6 +31,9 @@ from repomd import mdErrors
 from repomd import packageSack
 from packages import YumAvailablePackage
 import mdcache
+import parser
+
+_is_fnmatch_pattern = re.compile(r"[*?[]").search
 
 class YumPackageSack(packageSack.PackageSack):
     """imports/handles package objects from an mdcache dict object"""
@@ -102,19 +105,24 @@ class RepoStorage:
             self.sqlite = True
             self.sqlitecache = sqlitecache
             
+        self._selectSackType()
+    
+    def _selectSackType(self):
+
         if (self.sqlite):
+            import sqlitecache
+            import sqlitesack
             self.pkgSack = sqlitesack.YumSqlitePackageSack(sqlitesack.YumAvailablePackageSqlite)
         else:
             self.pkgSack = YumPackageSack(YumAvailablePackage)
-            
-    
+        
     def __str__(self):
-        return self.repos.keys()
+        return str(self.repos.keys())
     
 
     def add(self, repoobj):
         if self.repos.has_key(repoobj.id):
-            raise Errors.RepoError, 'Repository %s already added, not adding again' % (repoobj.id)
+            raise Errors.RepoError, 'Repository %s is listed more than once in the configuration' % (repoobj.id)
         self.repos[repoobj.id] = repoobj
         
 
@@ -134,16 +142,55 @@ class RepoStorage:
             raise Errors.RepoError, \
                 'Error getting repository data for %s, repository not found' % (repoid)
 
+    def findRepos(self,pattern):
+        """find all repositories matching fnmatch `pattern`"""
+
+        result = []
+        
+        for item in pattern.split(','):
+            item = item.strip()
+            match = re.compile(fnmatch.translate(item)).match
+            for name,repo in self.repos.items():
+                if match(name):
+                    result.append(repo)
+        return result
+        
     def disableRepo(self, repoid):
-        """disable a repository from use"""
-        thisrepo = self.getRepo(repoid)
-        thisrepo.disable()
-            
+        """disable a repository from use
+        
+        fnmatch wildcards may be used to disable a group of repositories.
+        returns repoid of disabled repos as list
+        """
+        repos = []
+        if _is_fnmatch_pattern(repoid) or repoid.find(',') != -1:
+            for repo in self.findRepos(repoid):
+                repos.append(repo.id)
+                repo.disable()
+        else:
+            thisrepo = self.getRepo(repoid)
+            repos.append(thisrepo.id)
+            thisrepo.disable()
+        
+        return repos
+        
     def enableRepo(self, repoid):
-        """disable a repository from use"""
-        thisrepo = self.getRepo(repoid)
-        thisrepo.enable()
-            
+        """enable a repository for use
+        
+        fnmatch wildcards may be used to enable a group of repositories.
+        returns repoid of enables repos as list
+        """
+        repos = []
+        if _is_fnmatch_pattern(repoid) or repoid.find(',') != -1:
+            for repo in self.findRepos(repoid):
+                repos.append(repo.id)
+                repo.enable()
+        else:
+            thisrepo = self.getRepo(repoid)
+            repos.append(thisrepo.id)
+            thisrepo.enable()
+        
+        return repos
+        
     def listEnabled(self):
         """return list of enabled repo objects"""
         returnlist = []
@@ -163,11 +210,17 @@ class RepoStorage:
         return returnlist
 
     def setCache(self, cacheval):
-        """set's cache value in all repos"""
+        """sets cache value in all repos"""
         self.cache = cacheval
         for repo in self.repos.values():
             repo.cache = cacheval
 
+    def setCacheDir(self, cachedir):
+        """sets the cachedir value in all repos"""
+
+        for repo in self.repos.values():
+            repo.basecachedir = cachedir
+
     def setProgressBar(self, obj):
         """set's the progress bar for downloading files from repos"""
         
@@ -198,8 +251,11 @@ class RepoStorage:
         else:
             if type(which) == types.ListType:
                 for repo in which:
-                    repobj = self.getRepo(repo)
-                    myrepos.append(repobj)
+                    if isinstance(repo, Repository):
+                        myrepos.append(repo)
+                    else:
+                        repobj = self.getRepo(repo)
+                        myrepos.append(repobj)
             elif type(which) == types.StringType:
                 repobj = self.getRepo(which)
                 myrepos.append(repobj)
@@ -269,8 +325,8 @@ class Repository:
         # some default (ish) things
         self.urls = []
         self.gpgcheck = 0
-        self.enabled = 1
-        self.enablegroups = 1
+        self.enabled = 0
+        self.enablegroups = 0
         self.groupsfilename = 'yumgroups.xml' # something some freaks might 
                                               # eventually want
         self.setkeys = []
@@ -279,16 +335,19 @@ class Repository:
         self.cache = 0
         self.callback = None # callback for the grabber
         self.failure_obj = None
-        self.mirrorlistfn = None # filename/url of mirrorlist file
+        self.mirrorlist = None # filename/url of mirrorlist file
         self.mirrorlistparsed = 0
-        self.baseurls = [] # baseurls from the config file
+        self.baseurl = [] # baseurls from the config file
         self.yumvar = {} # empty dict of yumvariables for $string replacement
         self.proxy_password = None
         self.proxy_username = None
         self.proxy = None
-        self.proxy_dict = {}
+        self._proxy_dict = {}
+        self.metadata_cookie_fn = 'cachecookie'
+        self.groups_added = False
         
         # throw in some stubs for things that will be set by the config class
+        self.basecachedir = ""
         self.cachedir = ""
         self.pkgdir = ""
         self.hdrdir = ""
@@ -347,8 +406,8 @@ class Repository:
         output = '[%s]\n' % self.id
         vars = ['name', 'bandwidth', 'enabled', 'enablegroups', 
                  'gpgcheck', 'includepkgs', 'keepalive', 'proxy',
-                 'proxy_password', 'proxy_username', 'excludes', 
-                 'retries', 'throttle', 'timeout', 'mirrorlistfn', 
+                 'proxy_password', 'proxy_username', 'exclude', 
+                 'retries', 'throttle', 'timeout', 'mirrorlist', 
                  'cachedir', 'gpgkey', 'pkgdir', 'hdrdir']
         vars.sort()
         for attr in vars:
@@ -388,10 +447,10 @@ class Repository:
         return self.setkeys
     
     def doProxyDict(self):
-        if self.proxy_dict:
+        if self._proxy_dict:
             return
         
-        self.proxy_dict = {} # zap it
+        self._proxy_dict = {} # zap it
         proxy_string = None
         if self.proxy not in [None, '_none_']:
             proxy_string = '%s' % self.proxy
@@ -409,9 +468,9 @@ class Repository:
                               proxy_host, proxy_rest)
                                                  
         if proxy_string is not None:
-            self.proxy_dict['http'] = proxy_string
-            self.proxy_dict['https'] = proxy_string
-            self.proxy_dict['ftp'] = proxy_string
+            self._proxy_dict['http'] = proxy_string
+            self._proxy_dict['https'] = proxy_string
+            self._proxy_dict['ftp'] = proxy_string
 
     def setupGrab(self):
         """sets up the grabber functions with the already stocked in urls for
@@ -423,17 +482,12 @@ class Repository:
             mgclass = urlgrabber.mirror.MirrorGroup
         
         
-        self.doProxyDict()
-        prxy = None
-        if self.proxy_dict:
-            prxy = self.proxy_dict
-        
         self.grabfunc = URLGrabber(keepalive=self.keepalive, 
                                    bandwidth=self.bandwidth,
                                    retry=self.retries,
                                    throttle=self.throttle,
                                    progress_obj=self.callback,
-                                   proxies = prxy,
+                                   proxies = self.proxy_dict,
                                    failure_callback=self.failure_obj,
                                    timeout=self.timeout,
                                    reget='simple')
@@ -442,6 +496,16 @@ class Repository:
 
     def dirSetup(self):
         """make the necessary dirs, if possible, raise on failure"""
+
+        cachedir = os.path.join(self.basecachedir, self.id)
+        pkgdir = os.path.join(cachedir, 'packages')
+        hdrdir = os.path.join(cachedir, 'headers')
+        self.set('cachedir', cachedir)
+        self.set('pkgdir', pkgdir)
+        self.set('hdrdir', hdrdir)
+        cookie = self.cachedir + '/' + self.metadata_cookie_fn
+        self.set('metadata_cookie', cookie)
+
         for dir in [self.cachedir, self.hdrdir, self.pkgdir]:
             if self.cache == 0:
                 if os.path.exists(dir) and os.path.isdir(dir):
@@ -462,15 +526,15 @@ class Repository:
            with valid ones, run  self.check() at the end to make sure it worked"""
 
         goodurls = []
-        if self.mirrorlistfn and not self.mirrorlistparsed:
-            mirrorurls = getMirrorList(self.mirrorlistfn)
+        if self.mirrorlist and not self.mirrorlistparsed:
+            mirrorurls = getMirrorList(self.mirrorlist, self.proxy_dict)
             self.mirrorlistparsed = 1
             for url in mirrorurls:
-                url = variableReplace(self.yumvar, url)
-                self.baseurls.append(url)
-        
-        for url in self.baseurls:
-            url = variableReplace(self.yumvar, url)
+                url = parser.varReplace(url, self.yumvar)
+                self.baseurl.append(url)
+
+        for url in self.baseurl:
+            url = parser.varReplace(url, self.yumvar)
             (s,b,p,q,f,o) = urlparse.urlparse(url)
             if s not in ['http', 'ftp', 'file', 'https']:
                 print 'not using ftp, http[s], or file for repos, skipping - %s' % (url)
@@ -483,7 +547,7 @@ class Repository:
         self.setupGrab() # update the grabber for the urls
 
     def get(self, url=None, relative=None, local=None, start=None, end=None,
-            copy_local=0, checkfunc=None, text=None, reget='simple'):
+            copy_local=0, checkfunc=None, text=None, reget='simple', cache=True):
         """retrieve file from the mirrorgroup for the repo
            relative to local, optionally get range from
            start to end, also optionally retrieve from a specific baseurl"""
@@ -494,6 +558,11 @@ class Repository:
         # if url is None do a grab via the mirror group/grab for the repo
         # return the path to the local file
 
+        if cache:
+            headers = None
+        else:
+            headers = (('Pragma', 'no-cache'),)
+
         if local is None or relative is None:
             raise Errors.RepoError, \
                   "get request for Repo %s, gave no source or dest" % self.id
@@ -503,17 +572,14 @@ class Repository:
             self.failure_obj = (f_func, f_args, f_kwargs)
         
         if self.cache == 1:
-            if os.path.exists(local):
-                return local
+            if os.path.exists(local): # FIXME - we should figure out a way
+                return local          # to run the checkfunc from here
+                
             else: # ain't there - raise
                 raise Errors.RepoError, \
                     "Caching enabled but no local cache of %s from %s" % (local,
                            self)
 
-        self.doProxyDict()
-        prxy = None
-        if self.proxy_dict:
-            prxy = self.proxy_dict
         if url is not None:
             ug = URLGrabber(keepalive = self.keepalive, 
                             bandwidth = self.bandwidth,
@@ -522,25 +588,20 @@ class Repository:
                             progres_obj = self.callback,
                             copy_local = copy_local,
                             reget = reget,
-                            proxies = prxy,
+                            proxies = self.proxy_dict,
                             failure_callback = self.failure_obj,
-                            timeout = self.timeout,
-                            checkfunc = checkfunc)
+                            timeout=self.timeout,
+                            checkfunc=checkfunc,
+                            http_headers=headers,
+                            )
             
             remote = url + '/' + relative
 
             try:
                 result = ug.urlgrab(remote, local,
-                                    text = text,
-                                    range = (start, end), 
-                                    retry = self.retries,
-                                    copy_local = copy_local,
-                                    reget = reget,
-                                    proxies = prxy,
-                                    failure_callback = self.failure_obj,
-                                    timeout = self.timeout,
-                                    checkfunc = checkfunc)
-                                    
+                                    text=text,
+                                    range=(start, end), 
+                                    )
             except URLGrabError, e:
                 raise Errors.RepoError, \
                     "failed to retrieve %s from %s\nerror was %s" % (relative, self.id, e)
@@ -552,13 +613,41 @@ class Repository:
                                            range = (start, end),
                                            copy_local=copy_local,
                                            reget = reget,
-                                           checkfunc=checkfunc)
+                                           checkfunc=checkfunc,
+                                           http_headers=headers,
+                                           )
             except URLGrabError, e:
                 raise Errors.RepoError, "failure: %s from %s: %s" % (relative, self.id, e)
                 
         return result
            
         
+    def metadataCurrent(self):
+        """Check if there is a metadata_cookie and check its age. If the 
+        age of the cookie is less than metadata_expire time then return true
+        else return False"""
+        
+        val = False
+        if os.path.exists(self.metadata_cookie):
+            cookie_info = os.stat(self.metadata_cookie)
+            if cookie_info[8] + self.metadata_expire > time.time():
+                val = True
+        
+        return val
+
+    
+    def setMetadataCookie(self):
+        """if possible, set touch the metadata_cookie file"""
+        
+        check = self.metadata_cookie
+        if not os.path.exists(self.metadata_cookie):
+            check = self.cachedir
+        
+        if os.access(check, os.W_OK):
+            fo = open(self.metadata_cookie, 'w+')
+            fo.close()
+            del fo
+            
     def getRepoXML(self, text=None):
         """retrieve/check/read in repomd.xml from the repository"""
 
@@ -566,25 +655,56 @@ class Repository:
         local = self.cachedir + '/repomd.xml'
         if self.repoXML is not None:
             return
-            
-        if self.cache == 1:
+
+        if self.cache or self.metadataCurrent():
             if not os.path.exists(local):
                 raise Errors.RepoError, 'Cannot find repomd.xml file for %s' % (self)
             else:
                 result = local
         else:
+            checkfunc = (self._checkRepoXML, (), {})
             try:
-                result = self.get(relative=remote, local=local,
-                                  copy_local=1, text=text, reget=None)
+                result = self.get(relative=remote,
+                                  local=local,
+                                  copy_local=1,
+                                  text=text,
+                                  reget=None,
+                                  checkfunc=checkfunc,
+                                  cache=self.http_caching == 'all')
+
             except URLGrabError, e:
                 raise Errors.RepoError, 'Error downloading file %s: %s' % (local, e)
+            
+            # if we have a 'fresh' repomd.xml then update the cookie
+            self.setMetadataCookie()
 
         try:
             self.repoXML = repoMDObject.RepoMD(self.id, result)
         except mdErrors.RepoMDError, e:
             raise Errors.RepoError, 'Error importing repomd.xml from %s: %s' % (self, e)
-
     
+    def _checkRepoXML(self, fo):
+        if type(fo) is types.InstanceType:
+            filepath = fo.filename
+        else:
+            filepath = fo
+        
+        try:
+            foo = repoMDObject.RepoMD(self.id, filepath)
+        except mdErrors.RepoMDError, e:
+            raise URLGrabError(-1, 'Error importing repomd.xml for %s: %s' % (self, e))
+
+
+    def __getProxyDict(self):
+        self.doProxyDict()
+        if self._proxy_dict:
+            return self._proxy_dict
+        return None
+
+    # consistent access to how proxy information should look (and ensuring
+    # that it's actually determined for the repo)
+    proxy_dict = property(__getProxyDict)
+
     def _checkMD(self, fn, mdtype):
         """check the metadata type against its checksum"""
         
@@ -660,7 +780,8 @@ class Repository:
         try:
             checkfunc = (self._checkMD, (mdtype,), {})
             local = self.get(relative=remote, local=local, copy_local=1,
-                             checkfunc=checkfunc, reget=None)
+                             checkfunc=checkfunc, reget=None, 
+                             cache=self.http_caching == 'all')
         except URLGrabError, e:
             raise Errors.RepoError, \
                 "Could not retrieve %s matching remote checksum from %s" % (local, self)
@@ -697,7 +818,7 @@ class Repository:
         
         
 
-def getMirrorList(mirrorlist):
+def getMirrorList(mirrorlist, pdict = None):
     """retrieve an up2date-style mirrorlist file from a url, 
        we also s/$ARCH/$BASEARCH/ and move along
        returns a list of the urls from that file"""
@@ -715,7 +836,7 @@ def getMirrorList(mirrorlist):
         url = mirrorlist
 
     try:
-        fo = urlresolver.urlopen(url)
+        fo = urlresolver.urlopen(url, proxies=pdict)
     except urlgrabber.grabber.URLGrabError, e:
         fo = None
 
@@ -730,42 +851,3 @@ def getMirrorList(mirrorlist):
     
     return returnlist
 
-def variableReplace(yumvar, thing):
-    """ do the replacement of $ variables, releasever, arch and basearch on any 
-        string or list  passed to it - returns whatever you passed"""
-    
-    if thing is None:
-        return thing
-
-    if type(thing) is types.StringType:
-        shortlist = []
-        shortlist.append(thing)
-        
-    if type(thing) is types.ListType:
-        shortlist = thing
-    
-    basearch_reg = re.compile('\$basearch', re.I)
-    arch_reg = re.compile('\$arch', re.I)
-    releasever_reg = re.compile('\$releasever', re.I)
-    yumvar_reg = {}
-
-    for num in range(0,10):
-        env = '\$YUM%s' % num
-        yumvar_reg[num] = re.compile(env, re.I)
-
-    returnlist = []        
-    for string in shortlist:
-        (string, count) = basearch_reg.subn(yumvar['basearch'], string)
-        (string, count) = arch_reg.subn(yumvar['arch'], string)
-        (string, count) = releasever_reg.subn(yumvar['releasever'], string)
-        for num in range(0,10):
-            (string, count) = yumvar_reg[num].subn(yumvar[num], string)
-        returnlist.append(string)
-        
-    if type(thing) is types.StringType:
-        thing = returnlist[0]
-    
-    if type(thing) is types.ListType:
-        thing = returnlist
-        
-    return thing
