@@ -24,7 +24,7 @@ import os.path
 import types
 import repos
 from packages import YumAvailablePackage
-from repomd import mdUtils
+from repomd import mdUtils, mdErrors
 
 # Simple subclass of YumAvailablePackage that can load 'simple headers' from
 # the database when they are requested
@@ -71,7 +71,7 @@ class YumAvailablePackageSqlite(YumAvailablePackage):
 
     def returnPrco(self, prcotype):
         if not self.prco[prcotype]:
-           self.prco = self.sack.getPrco(self.pkgId)
+           self.prco = self.sack.getPrco(self.pkgId, prcotype)
         return self.prco[prcotype]
 
 class YumSqlitePackageSack(repos.YumPackageSack):
@@ -126,28 +126,35 @@ class YumSqlitePackageSack(repos.YumPackageSack):
         result = []
         for (rep,cache) in self.otherdb.items():
             cur = cache.cursor()
-            cur.execute("select * from packages,changelog where packages.pkgId = %s and packages.pkgKey = changelog.pkgKey",pkgId)
+            cur.execute("select changelog.date as date,\
+                changelog.author as author,\
+                changelog.changelog as changelog from packages,changelog where packages.pkgId = %s and packages.pkgKey = changelog.pkgKey",pkgId)
             for ob in cur.fetchall():
-                result.append(( ob['changelog.date'],
-                                ob['changelog.author'],
-                                ob['changelog.changelog']
+                result.append(( ob['date'],
+                                ob['author'],
+                                ob['changelog']
                               ))
         return result
 
-    def getPrco(self, pkgId):
-        result = {}
+    def getPrco(self,pkgId, prcotype=None):
+        if prcotype is not None:
+            result = {'requires': [], 'provides': [], 'obsoletes': [], 'conflicts': []}
+        else:
+            result = { prcotype: [] }
         for (rep, cache) in self.primarydb.items():
             cur = cache.cursor()
-            for prco in ['requires', 'provides', 'obsoletes', 'conflicts']:
-                result[prco] = []
-                cur.execute("select * from packages,%s where packages.pkgId = %s and packages.pkgKey = %s.pkgKey", prco, pkgId, prco)
+            for prco in result.keys():
+                cur.execute("select %s.name as name, %s.version as version,\
+                    %s.release as release, %s.epoch as epoch, %s.flags as flags\
+                    from packages,%s\
+                    where packages.pkgId = %s and packages.pkgKey = %s.pkgKey", prco, prco, prco, prco, prco, prco, pkgId, prco)
                 for ob in cur.fetchall():
-                    name = ob['%s.name' % prco ]
-                    version = ob['%s.version' % prco ]
-                    release = ob['%s.release' % prco ]
-                    epoch = ob['%s.epoch' % prco ]
-                    flags = ob['%s.flags' % prco ]
-                    result[prco].append((name, flags, (version, release, epoch)))
+                    name = ob['name']
+                    version = ob['version']
+                    release = ob['release']
+                    epoch = ob['epoch']
+                    flags = ob['flags']
+                    result[prco].append((name, flags, (epoch, version, release)))
         return result
 
     # Get all files for a certain pkgId from the filelists.xml metadata
@@ -156,14 +163,20 @@ class YumSqlitePackageSack(repos.YumPackageSack):
             found = False
             result = {}
             cur = cache.cursor()
-            cur.execute("select * from packages,filelist where packages.pkgId = %s and packages.pkgKey = filelist.pkgKey", pkgId)
+            cur.execute("select filelist.dirname as dirname,\
+                filelist.filetypes as filetypes,\
+                filelist.filenames as filenames from packages,filelist\
+                where packages.pkgId = %s and packages.pkgKey = filelist.pkgKey", pkgId)
             for ob in cur.fetchall():
                 found = True
-                dirname = ob['filelist.dirname']
-                filetypes = ob['filelist.filetypes'].split('/')
-                filenames = ob['filelist.filenames'].split('/')
+                dirname = ob['dirname']
+                filetypes = decodefiletypelist(ob['filetypes'])
+                filenames = decodefilenamelist(ob['filenames'])
                 while(filenames):
-                    filename = dirname+'/'+filenames.pop()
+                    if dirname:
+                        filename = dirname+'/'+filenames.pop()
+                    else:
+                        filename = filenames.pop()
                     filetype = filetypes.pop()
                     result.setdefault(filetype,[]).append(filename)
             if (found):
@@ -172,64 +185,112 @@ class YumSqlitePackageSack(repos.YumPackageSack):
             
     # Search packages that either provide something containing name
     # or provide a file containing name 
-    def searchAll(self,name):
+    def searchAll(self, name, query_type='like'):
+        
         # This should never be called with a name containing a %
         assert(name.find('%') == -1)
-        result = []
+        result = {}
         quotename = name.replace("'","''")
+        (dirname,filename) = os.path.split(name)
+        
+        # check provides
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            cur.execute("select DISTINCT packages.pkgId from provides,packages where provides.name LIKE '%%%s%%' AND provides.pkgKey = packages.pkgKey" % quotename)
+            cur.execute("select DISTINCT packages.pkgId as pkgId from provides,packages where provides.name LIKE '%%%s%%' AND provides.pkgKey = packages.pkgKey" % quotename)
             for ob in cur.fetchall():
-                if (self.excludes[rep].has_key(ob['packages.pkgId'])):
+                if (self.excludes[rep].has_key(ob['pkgId'])):
                     continue
-                pkg = self.getPackageDetails(ob['packages.pkgId'])
-                result.append((self.pc(pkg,rep)))
 
+                pkgid = ob['pkgId']
+                if not result.has_key(pkgid):
+                    pkg = self.getPackageDetails(ob['pkgId'])
+                    result[pkgid] = (self.pc(pkg,rep))
+        
+        # check filelists/dirlists
         for (rep,cache) in self.filelistsdb.items():
-            cur = cache.cursor()
-            (dir,filename) = os.path.split(quotename)
-            # This query means:
-            # Either name is a substring of dirname or the directory part
-            # in name is a substring of dirname and the file part is part
-            # of filelist
-            cur.execute("select * from packages,filelist where \
-                (filelist.dirname LIKE '%%%s%%' \
-                OR (filelist.dirname LIKE '%%%s%%' AND\
-                filelist.filenames LIKE '%%%s%%'))\
-                AND (filelist.pkgKey = packages.pkgKey)" % (quotename,dir,filename))
-            for ob in cur.fetchall():
-                # Check if it is an actual match
-                # The query above can give false positives, when
-                # a package provides /foo/aaabar it will also match /foo/bar
-                if (self.excludes[rep].has_key(ob['packages.pkgId'])):
-                    continue
-                real = False
-                for filename in ob['filelist.filenames'].split('/'):
-                    if (ob['filelist.dirname']+'/'+filename).find(name) != -1:
-                        real = True
-                if (not real):
-                    continue
-                pkg = self.getPackageDetails(ob['packages.pkgId'])
-                result.append((self.pc(pkg,rep)))
-        return result     
+            querystrings = []
+            # dirnames
+            # just the dirname
+            if dirname != '':
+                tmp = "select packages.pkgId as pkgId,\
+                filelist.dirname as dirname,\
+                filelist.filetypes as filetypes,\
+                filelist.filenames as filenames \
+                from packages,filelist where \
+                filelist.dirname LIKE '%%%s%%' \
+                AND (filelist.pkgKey = packages.pkgKey)" % (dirname)
+                querystrings.append(tmp)
+                
+            # look at full quotename
+            tmp = "select packages.pkgId as pkgId,\
+                filelist.dirname as dirname,\
+                filelist.filetypes as filetypes,\
+                filelist.filenames as filenames \
+                from packages,filelist where \
+                filelist.dirname LIKE '%%%s%%' \
+                AND (filelist.pkgKey = packages.pkgKey)" % (quotename)
+            querystrings.append(tmp)
+
+            # filenames
+            tmp = "select packages.pkgId as pkgId,\
+                filelist.dirname as dirname,\
+                filelist.filetypes as filetypes,\
+                filelist.filenames as filenames \
+                from packages,filelist where \
+                filelist.filenames LIKE '%%%s%%'\
+                AND (filelist.pkgKey = packages.pkgKey)" % (filename)
+            
+            querystrings.append(tmp)
+        
+            for querystring in querystrings:
+                cur = cache.cursor()
+                cur.execute("%s" % querystring)
+
+                # cull the results for false positives
+                for ob in cur.fetchall():
+                    # Check if it is an actual match
+                    # The query above can give false positives, when
+                    # a package provides /foo/aaabar it will also match /foo/bar
+                    if (self.excludes[rep].has_key(ob['pkgId'])):
+                        continue
+                    real = False
+        
+                    for filename in decodefilenamelist(ob['filenames']):
+                        if (ob['dirname'] + '/' + filename).find(name) != -1:
+                            real = True
+                    if (not real):
+                        continue
+
+                    pkgid = ob['pkgId']
+                    if not result.has_key(pkgid):
+                        pkg = self.getPackageDetails(ob['pkgId'])
+                        result[pkgid] = (self.pc(pkg,rep))
+
+        return result.values()
     
     def returnObsoletes(self):
         obsoletes = {}
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            cur.execute("select * from obsoletes,packages where obsoletes.pkgKey = packages.pkgKey")
+            cur.execute("select packages.name as name,\
+                packages.pkgId as pkgId,\
+                packages.arch as arch, packages.epoch as epoch,\
+                packages.release as release, packages.version as version,\
+                obsoletes.name as oname, obsoletes.epoch as oepoch,\
+                obsoletes.release as orelease, obsoletes.version as oversion,\
+                obsoletes.flags as oflags\
+                from obsoletes,packages where obsoletes.pkgKey = packages.pkgKey")
             for ob in cur.fetchall():
                 # If the package that is causing the obsoletes is excluded
                 # continue without processing the obsoletes
-                if (self.excludes[rep].has_key(ob['packages.pkgId'])):
+                if (self.excludes[rep].has_key(ob['pkgId'])):
                     continue
-                key = ( ob['packages.name'],ob['packages.arch'],
-                        ob['packages.epoch'],ob['packages.version'],
-                        ob['packages.release'])
-                (n,f,e,v,r) = ( ob['obsoletes.name'],ob['obsoletes.flags'],
-                                ob['obsoletes.epoch'],ob['obsoletes.version'],
-                                ob['obsoletes.release'])
+                key = ( ob['name'],ob['arch'],
+                        ob['epoch'],ob['version'],
+                        ob['release'])
+                (n,f,e,v,r) = ( ob['oname'],ob['oflags'],
+                                ob['oepoch'],ob['oversion'],
+                                ob['orelease'])
 
                 obsoletes.setdefault(key,[]).append((n,f,(e,v,r)))
 
@@ -242,16 +303,15 @@ class YumSqlitePackageSack(repos.YumPackageSack):
             for ob in cur.fetchall():
                 pkg = self.db2class(ob)
                 return pkg
-    
-    def searchProvides(self, name):
-        """return list of package providing the name (any evr and flag)"""
-        provides = []
-        # First search the provides cache
+
+    def searchPrco(self, name, prcotype):
+        """return list of packages having prcotype name (any evr and flag)"""
+        results = []
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            cur.execute("select * from provides where name = %s" , (name))
-            provs = cur.fetchall()
-            for res in provs:
+            cur.execute("select * from %s where name = %s" , (prcotype, name))
+            prcos = cur.fetchall()
+            for res in prcos:
                 cur.execute("select * from packages where pkgKey = %s" , (res['pkgKey']))
                 for x in cur.fetchall():
                     pkg = self.db2class(x)
@@ -260,7 +320,7 @@ class YumSqlitePackageSack(repos.YumPackageSack):
                                             
                     # Add this provides to prco otherwise yum doesn't understand
                     # that it matches
-                    pkg.prco = {'provides': 
+                    pkg.prco = {prcotype: 
                       [
                       { 'name': res.name,
                         'flags': res.flags,
@@ -270,18 +330,19 @@ class YumSqlitePackageSack(repos.YumPackageSack):
                       }
                       ]
                     }
-                    provides.append(self.pc(pkg,rep))
+                    results.append(self.pc(pkg,rep))
 
-        # If it's not a filename, we are done
-        if (name.find('/') != 0):
-            return provides
+
+        # If it's not a provides or a filename, we are done
+        if (prcotype != "provides" or name.find('/') != 0):
+            return results
 
         # If it is a filename, search the primary.xml file info
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
             cur.execute("select * from files where name = %s" , (name))
-            provs = cur.fetchall()
-            for res in provs:
+            files = cur.fetchall()
+            for res in files:
                 cur.execute("select * from packages where pkgKey = %s" , (res['pkgKey']))
                 for x in cur.fetchall():
                     pkg = self.db2class(x)
@@ -289,28 +350,48 @@ class YumSqlitePackageSack(repos.YumPackageSack):
                         continue
                                             
                     pkg.files = {name: res['type']}
-                    provides.append(self.pc(pkg,rep))
+                    results.append(self.pc(pkg,rep))
 
-        # If it is a filename, search the primary.xml file info
+        # If it is a filename, search the files.xml file info
         for (rep,cache) in self.filelistsdb.items():
             cur = cache.cursor()
             (dirname,filename) = os.path.split(name)
-            cur.execute("select * from filelist,packages where dirname = %s AND filelist.pkgKey = packages.pkgKey" , (dirname))
-            provs = cur.fetchall()
-            for res in provs:
-                if (self.excludes[rep].has_key(res['packages.pkgId'])):
+            cur.execute("select packages.pkgId as pkgId,\
+                filelist.dirname as dirname,\
+                filelist.filetypes as filetypes,\
+                filelist.filenames as filenames \
+                from filelist,packages where dirname = %s AND filelist.pkgKey = packages.pkgKey" , (dirname))
+            files = cur.fetchall()
+            for res in files:
+                if (self.excludes[rep].has_key(res['pkgId'])):
                     continue
                                         
                 # If it matches the dirname, that doesnt mean it matches
                 # the filename, check if it does
                 if filename and \
-                  not filename in res['filelist.filenames'].split('/'):
+                  not filename in res['filenames'].split('/'):
                     continue
                 # If it matches we only know the packageId
-                pkg = self.getPackageDetails(res['packages.pkgId'])
-                provides.append(self.pc(pkg,rep))
-        return provides
+                pkg = self.getPackageDetails(res['pkgId'])
+                results.append(self.pc(pkg,rep))
+        return results
+
+    def searchProvides(self, name):
+        """return list of packages providing name (any evr and flag)"""
+        return self.searchPrco(name, "provides")
                 
+    def searchRequires(self, name):
+        """return list of packages requiring name (any evr and flag)"""
+        return self.searchPrco(name, "requires")
+
+    def searchObsoletes(self, name):
+        """return list of packages obsoleting name (any evr and flag)"""
+        return self.searchPrco(name, "obsoletes")
+
+    def searchConflicts(self, name):
+        """return list of packages conflicting with name (any evr and flag)"""
+        return self.searchPrco(name, "conflicts")
+
     # TODO this seems a bit ugly and hackish
     def db2class(self,db,nevra_only=False):
       class tmpObject:
@@ -322,7 +403,7 @@ class YumSqlitePackageSack(repos.YumPackageSack):
       if (nevra_only):
         return y
       y.hdrange = {'start': db.rpm_header_start,'end': db.rpm_header_end}
-      y.location = {'href': db.location_href,'value':''}
+      y.location = {'href': db.location_href,'value': '', 'base': db.location_base}
       y.checksum = {'pkgid': 'YES','type': db.checksum_type, 
                     'value': db.checksum_value }
       y.time = {'build': db.time_build, 'file': db.time_file }
@@ -363,15 +444,32 @@ class YumSqlitePackageSack(repos.YumPackageSack):
                 if (self.excludes[rep].has_key(x.pkgId)):
                     continue                    
                 allpkg.append(self.pc(self.db2class(x,True),rep))
-        # Now find the newest one
-        newest = allpkg.pop()
-        for pkg in allpkg:
-            (e2, v2, r2) = newest.returnEVR()
-            (e,v,r) = pkg.returnEVR()
-            rc = mdUtils.compareEVR((e,v,r), (e2, v2, r2))
-            if (rc > 0):
-                newest = pkg
-        return newest
+        
+        # if we've got zilch then raise
+        if not allpkg:
+            raise mdErrors.PackageSackError, 'No Package Matching %s.%s' % naTup
+        return mdUtils.newestInList(allpkg)
+
+    def returnNewestByName(self, name=None):
+        # If name is set do it from the database otherwise use our parent's
+        # returnNewestByName
+        if (not name):
+            return repos.YumPackageSack.returnNewestByName(self, name)
+
+        # First find all packages that fulfill name
+        allpkg = []
+        for (rep,cache) in self.primarydb.items():
+            cur = cache.cursor()
+            cur.execute("select pkgId,name,epoch,version,release,arch from packages where name=%s", name)
+            for x in cur.fetchall():
+                if (self.excludes[rep].has_key(x.pkgId)):
+                    continue                    
+                allpkg.append(self.pc(self.db2class(x,True),rep))
+        
+        # if we've got zilch then raise
+        if not allpkg:
+            raise mdErrors.PackageSackError, 'No Package Matching %s' % name
+        return mdUtils.newestInList(allpkg)
 
     def returnPackages(self, repoid=None):
         """Returns a list of packages, only containing nevra information """
@@ -389,10 +487,30 @@ class YumSqlitePackageSack(repos.YumPackageSack):
     def searchNevra(self, name=None, epoch=None, ver=None, rel=None, arch=None):        
         """return list of pkgobjects matching the nevra requested"""
         returnList = []
-        # Search all repositories
+        
+        # make sure some dumbass didn't pass us NOTHING to search on
+        empty = True
+        for arg in (name, epoch, ver, rel, arch):
+            if arg:
+                empty = False
+        if empty:
+            return returnList
+        
+        # make up our execute string
+        q = "select * from packages WHERE"
+        for (col, var) in [('name', name), ('epoch', epoch), ('version', ver),
+                           ('arch', arch), ('release', rel)]:
+            if var:
+                if q[-5:] != 'WHERE':
+                    q = q + ' AND %s = "%s"' % (col, var)
+                else:
+                    q = q + ' %s = "%s"' % (col, var)
+            
+        # Search all repositories            
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            cur.execute("select * from packages WHERE name = %s AND epoch = %s AND version = %s AND release = %s AND arch = %s" , (name,epoch,ver,rel,arch))
+            #cur.execute("select * from packages WHERE name = %s AND epoch = %s AND version = %s AND release = %s AND arch = %s" , (name,epoch,ver,rel,arch))
+            cur.execute(q)
             for x in cur.fetchall():
                 if (self.excludes[rep].has_key(x.pkgId)):
                     continue
@@ -414,3 +532,28 @@ class YumSqlitePackageSack(repos.YumPackageSack):
                 obj = self.pc(self.db2class(x), rep)
                 self.delPackage(obj)
 
+# Simple helper functions
+
+# Return a string representing filenamelist (filenames can not contain /)
+def encodefilenamelist(filenamelist):
+    return '/'.join(filenamelist)
+
+# Return a list representing filestring (filenames can not contain /)
+def decodefilenamelist(filenamestring):
+    return filenamestring.split('/')
+
+# Return a string representing filetypeslist
+# filetypes should be file, dir or ghost
+def encodefiletypelist(filetypelist):
+    result = ''
+    ft2string = {'file': 'f','dir': 'd','ghost': 'g'}
+    for x in filetypelist:
+        result += ft2string[x]
+    return result
+
+# Return a list representing filetypestring
+# filetypes should be file, dir or ghost
+def decodefiletypelist(filetypestring):
+    string2ft = {'f':'file','d': 'dir','g': 'ghost'}
+    return [string2ft[x] for x in filetypestring]
+
